/*
DV decoder module (libavcodec wrapper) for lvstream.
lvstream (C) 2005 Martin J. Fiedler <martin.fiedler@gmx.net>

lvstream is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

lvstream is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lvstream; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "main.h"
#include "options.h"
#include "dvdecode.h"

#include "avformat.h"
#include "avcodec.h"
#include "dv.h"

static DVDemuxContext* dv_demux=NULL;
static AVFormatContext* ic=NULL;
static AVCodecContext* coco=NULL;
static AVCodec* codec=NULL;
static AVFrame* avframe=NULL;


int dvdecode_init(void) {
  avcodec_init();
  register_avcodec(&dvvideo_decoder);
  ff_dv_init();

  ic=av_alloc_format_context();
  if(!ic) {
    printf("dvdecode_init(): cannot create lavf context\n");
    return 1;
  }

  dv_demux=dv_init_demux(ic);
  if(!dv_demux) {
    printf("dvdecode_init(): cannot create DV demuxer\n");
    return 1;
  }

  coco=&ic->streams[0]->codec;
  codec=avcodec_find_decoder(coco->codec_id);
  if(!codec) {
    printf("dvdecode_init(): cannot find DV decoder\n");
    return 1;
  }

  if(avcodec_open(coco,codec)) {
    printf("dvdecode_init(): cannot open DV decoder\n");
    return 1;
  }

  avframe=avcodec_alloc_frame();

  return 0;
}


int dvdecode_close(void) {
  avcodec_close(coco);
  av_free(avframe);
  return 0;
}


int dvdecode_extract_audio(uint8_t *frame, audio_block *ab) {
  AVPacket pkt;
  int size;

  // feed DV frame into demuxer
  size=dv_produce_packet(dv_demux,&pkt,frame,DV_FRAMESIZE);

  // read resulting packets until first audio stream is found
  while(size && pkt.stream_index!=1)
    size=dv_get_packet(dv_demux,&pkt);

  // no packet found? we're screwed
  if(!size) {
    ab->sample_count=0;
    return 1;
  }

  // produce a nice, clean audio block
  ab->sample_count=pkt.size/4;
  if(ab->sample_count>MAX_AUDIO_BLOCK_SIZE) ab->sample_count=MAX_AUDIO_BLOCK_SIZE;
  memcpy(ab->samples,pkt.data,ab->sample_count*4);
  return 0;
}


int dvdecode_decode_video(uint8_t *frame, frame_data *fd) {
  int got_picture,res;

  // let lavc decode the frame
  res=avcodec_decode_video(coco,avframe,&got_picture,frame,DV_FRAMESIZE);
  if(res<0 || !got_picture) {
    fd->Y=fd->U=fd->V=NULL;
    fd->Ystride=fd->Cstride=0;
    return 1;
  }

  // fill in the structure
  fd->Y=avframe->data[0];
  fd->U=avframe->data[1];
  fd->V=avframe->data[2];
  fd->Ystride=avframe->linesize[0];
  fd->Cstride=avframe->linesize[1];
  return 0;
}
