/*
Audio encoder module (LAME wrapper) for lvstream.
lvstream (C) 2005 Martin J. Fiedler <martin.fiedler@gmx.net>

lvstream is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

lvstream is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lvstream; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __LAMEWRAP_H__
#define __LAMEWRAP_H__

#include <stdint.h>
#include "lame.h"

#define LW_BUFSIZE (1<<13)

typedef struct {
//public:
  int in_rate;              // [in] input stream sample rate
  int in_channels;          // [in] input stream channels (1 or 2)
  int out_rate;             // [in] output stream sample rate
  int out_channels;         // [in] output stream channels (1 or 2)
  int bitrate;              // [in] desired bitrate in kbps
  int quality;              // [in] LAME quality setting
  int frameno;              // [out] number of frames fetched so far
  int in_framelength;       // [out] number of input PCM samples per frame
  int out_framelength;      // [out] number of output PCM samples per frame
  int framesize;            // [out] expected average MP3 frame size
//private:
  uint8_t buf[LW_BUFSIZE];
  lame_global_flags *lame;
  int rpos,wpos;
  int subsample;
  uint16_t *ssbuf;  // subsample buffer
} lw_t;


// initialize the private members of a LW instance
void lw_init(lw_t* lw);

// encode some samples (1 second maximum) into the buffer
// WARNING: the buffer size is rather small (about 10-20 frames), so you
//          should encode chunks of ~2000 samples and read frames often
void lw_encode(lw_t* lw, int16_t *samples, int count);

// encode directly. this may overwrite the sample buffer.
void lw_encode_direct(lw_t* lw, int16_t *samples, int count);

// finish encoding. this places some additional queued frames into the output
// which can be fetched afterwards
void lw_done(lw_t* lw);

// get one MP3 frame. this returns the starting address and length of one
// complete(!) MP3 frame inside LW's internal buffer
uint8_t* lw_get_frame(lw_t* lw, int *length);

// get as many complete MP3 frames as present in the buffer
uint8_t* lw_get_frames(lw_t* lw, int *length, int *count);

#endif/*__LAMEWRAP_H__*/
