/*
lvstream main program
lvstream (C) 2005 Martin J. Fiedler <martin.fiedler@gmx.net>

lvstream is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

lvstream is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lvstream; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "main.h"
#include "options.h"
#include "dvdecode.h"
#include "capture.h"
#include "audio.h"
#include "video.h"
#include "stream.h"
#include "sprite.h"

void* fast_memcpy(void *to, const void *from, size_t len) { return memcpy(to,from,len); }

int Running=1;

static void cchandler(int signum) {
  Running=0;
}

static void usrhandler(int signum) {
  read_sprite();
}


double get_time(void) {
  struct timeval tv;
  gettimeofday(&tv,NULL);
  return tv.tv_sec+0.000001*tv.tv_usec;
}


int main(int argc, char *argv[]) {
  double bench_time=0.0;

  printf("\e[1mKeyJ's Live Stream Provider, version %s (%s)\e[0m (%s capture)\n",VERSION,SYSTEM,CAPTURE_MODULE);
  set_default_options();           // load default options
  if(preparse_cmdline(argc,argv))  // search for config_file or help option
    return 0;
  load_config();                   // load config file
  if(parse_cmdline(argc,argv)) {   // really parse the command line
    printf("Use `%s -h' to get some help.\n",argv[0]);
    return 1;
  }

  printf("Initializing ...\n");
  if(dvdecode_init()) return 1;
  if(capture_init()) return 1;
  if(audio_init()) return 1;
  if(video_init()) return 1;
  if(sprite_init()) return 1;
  if(stream_init()) return 1;

  signal(SIGINT,cchandler);
  signal(SIGUSR1,usrhandler);

  printf("Starting Threads ...\n");
  if(audio_start()) goto fail;
  if(video_start()) goto fail;
  if(capture_start()) goto fail;

  printf("Capturing, press ^C to quit ...\n");
  if(benchmark) bench_time=get_time();
  while(Running) {
    printf("\r%d:%02d:%05.2f AU#%d | VBuf:%d Q:%d | drops: DV:%lld A:%d V:%d (%.1f%% = %.1f fps)  ",
      dv_frameno/FRAMERATE/3600,(dv_frameno/FRAMERATE/60)%60,
      (dv_frameno%(FRAMERATE*60))*1.0/FRAMERATE,
      lo_current_au,videobuf,me_quality,
      dv_drops,audio_drops,framedrops,
      (drops_last_second*100.0/FRAMERATE),
      FRAMERATE*(1.0-drops_last_second/(1.0*FRAMERATE))
    );
    fflush(stdout);
    usleep(1000000/FRAMERATE);
  }
  if(benchmark) bench_time=get_time()-bench_time;
  printf("\n");

  printf("Capture done, waiting for all threads to finish ...\n");
fail:
  capture_stop();
  audio_stop();
  video_stop();

  signal(SIGUSR1,SIG_DFL);

  if(benchmark)
    printf("BENCHMARK RESULTS: q=%d frames=%d time=%.1f fps=%.2f\n",
      me_quality,dv_frameno-1,bench_time,(dv_frameno-1)/bench_time);

  printf("All threads finished, cleaning up ...\n");
  capture_close();
  audio_close();
  video_close();
  stream_close();

  signal(SIGINT,SIG_DFL);
  printf("Done.\n");
  return 0;
}
