/*
OGM output module for lvstream.
lvstream (C) 2005 Martin J. Fiedler <martin.fiedler@gmx.net>

lvstream is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

lvstream is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lvstream; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __OGMWRITE_H__
#define __OGMWRITE_H__

#include <stdint.h>

typedef void (ogg_write_func_t) (uint8_t *data, int count, void *priv);

typedef struct {
  /* ogg_phys_stream_t */ void *phys_stream;
  uint64_t last_timestamp;
  uint32_t page_sequence_number;
  uint8_t header[27+255];
  uint8_t payload[255*255];
  int payload_size;
  int segment_count;
  int bos;
  int continuation;
} ogg_log_stream_t;

typedef struct {
  int page_size;                 // [in,opt] preferred page size
  ogg_write_func_t *write_func;  // [in] write function
  void *write_func_priv;         // [in,opt] private data for the write function (e.g. a FD)
  int offset;                    // [out] current output stream position
  // [private data follows]
  int log_stream_count;
  ogg_log_stream_t **log_streams;
} ogg_phys_stream_t;

void ogg_phys_stream_init(ogg_phys_stream_t* phys_stream);
void ogg_phys_stream_flush(ogg_phys_stream_t* phys_stream, int eos);
void ogg_phys_stream_free(ogg_phys_stream_t* phys_stream);

ogg_log_stream_t* ogg_log_stream_create(ogg_phys_stream_t* phys_stream, uint32_t bitstream_serial_number);
int ogg_log_stream_write(ogg_log_stream_t* log_stream, uint8_t *packet, int length, uint64_t timestamp);
int ogg_log_stream_flush(ogg_log_stream_t* log_stream, int eos);
void ogg_log_stream_free(ogg_log_stream_t* log_stream);

ogg_log_stream_t* ogm_video_stream_create(
  ogg_phys_stream_t* phys_stream,
  uint32_t bitstream_serial_number,
  char *fourcc,
  int rate_numerator, int rate_denominator,
  uint32_t width, uint32_t height,
  uint32_t buffersize);
ogg_log_stream_t* ogm_audio_stream_create(
  ogg_phys_stream_t* phys_stream,
  uint32_t bitstream_serial_number,
  char *fourcc,
  uint32_t sample_rate, uint8_t channels,
  uint32_t avg_bytes_per_sec,
  uint32_t buffersize, uint16_t block_align);

#endif/*__OGMWRITE_H__*/
