/*
Video encoding module (XviD wrapper) for lvstream.
lvstream (C) 2005 Martin J. Fiedler <martin.fiedler@gmx.net>

lvstream is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

lvstream is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lvstream; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include "xvid.h"
#include "xvidwrap.h"


static int me_presets[ME_PRESET_COUNT]={
  0,
  XVID_ME_HALFPELREFINE16,
  XVID_ME_HALFPELREFINE16 | XVID_ME_EXTSEARCH16,
  XVID_ME_HALFPELREFINE16 | XVID_ME_EXTSEARCH16 | XVID_ME_ADVANCEDDIAMOND16,
  XVID_ME_HALFPELREFINE16 | XVID_ME_HALFPELREFINE8 | XVID_ME_EXTSEARCH16 | XVID_ME_ADVANCEDDIAMOND16,
  XVID_ME_HALFPELREFINE16 | XVID_ME_HALFPELREFINE8 | XVID_ME_EXTSEARCH16 | XVID_ME_EXTSEARCH8 | XVID_ME_ADVANCEDDIAMOND16 | XVID_ME_ADVANCEDDIAMOND8,
  XVID_ME_HALFPELREFINE16 | XVID_ME_HALFPELREFINE8 | XVID_ME_EXTSEARCH16 | XVID_ME_EXTSEARCH8 | XVID_ME_ADVANCEDDIAMOND16 | XVID_ME_ADVANCEDDIAMOND8 | XVID_ME_CHROMA_PVOP,
  XVID_ME_HALFPELREFINE16 | XVID_ME_HALFPELREFINE8 | XVID_ME_EXTSEARCH16 | XVID_ME_EXTSEARCH8 | XVID_ME_USESQUARES16 | XVID_ME_USESQUARES8 | XVID_ME_CHROMA_PVOP
};
#define HALFPEL_SINCE 1
#define INTER4V_SINCE 4


char* xvid_init(void) {
  static xvid_gbl_info_t info;
  xvid_gbl_init_t init;

  // initialize XviD
  memset(&init,0,sizeof(xvid_gbl_init_t));
  init.version=XVID_VERSION;
  if(xvid_global(NULL,XVID_GBL_INIT,&init,NULL)<0) return NULL;

  // get XviD version
  memset(&info,0,sizeof(xvid_gbl_info_t));
  info.version=XVID_VERSION;
  if(xvid_global(NULL,XVID_GBL_INFO,&info,NULL)<0) return "unknown";
  return (char*)info.build;
}


void xvid_prepare(xvid_t *xvid,
                  int width, int height,
                  int bframes, int key_interval,
                  int rate_num, int rate_den
) {
  if(!xvid) return;
  xvid->handle=NULL;
  memset(xvid->plugin_data,0,sizeof(xvid->plugin_data));
  memset(&xvid->create,0,sizeof(xvid_enc_create_t));
  memset(&xvid->frame,0,sizeof(xvid_enc_frame_t));
  memset(&xvid->stats,0,sizeof(xvid_enc_stats_t));
  xvid->create.version=XVID_VERSION;
  xvid->create.width=width;
  xvid->create.height=height;
  xvid->create.profile=XVID_PROFILE_AS_L4;
  xvid->create.max_bframes=bframes;
  xvid->create.bquant_ratio=150;
  xvid->create.bquant_offset=100;
  xvid->create.max_key_interval=key_interval;
  xvid->create.global=XVID_GLOBAL_PACKED|XVID_GLOBAL_CLOSED_GOP;
  xvid->create.plugins=xvid->plugins;
  xvid->create.num_plugins=0;
  xvid->create.fincr=rate_den;
  xvid->create.fbase=rate_num;
  xvid->me_preset=4;
  xvid->frame.version=XVID_VERSION;
  xvid->frame.vol_flags=0;
  xvid->frame.vop_flags=XVID_VOP_HALFPEL|XVID_VOP_INTER4V;
  xvid->frame.motion=XVID_ME_ADVANCEDDIAMOND16|XVID_ME_ADVANCEDDIAMOND8|
                XVID_ME_HALFPELREFINE16|  XVID_ME_HALFPELREFINE8|
                XVID_ME_EXTSEARCH16|      XVID_ME_EXTSEARCH8|
                XVID_ME_CHROMA_PVOP;
  xvid->frame.quant=-1;
  xvid->frame.input.csp=XVID_CSP_PLANAR;
  xvid_set_me_preset(xvid,ME_PRESET_COUNT/2);
}

void xvid_add_plugin(xvid_t *xvid, xvid_plugin_func *func, void *param, int param_size) {
  int idx;
  if(!xvid || xvid->create.num_plugins>=MAX_PLUGINS) return;
  idx=xvid->create.num_plugins;
  if(param && param_size) {
    xvid->plugin_data[idx]=malloc(param_size);
    if(xvid->plugin_data[idx])
      memcpy(xvid->plugin_data[idx],param,param_size);
  }
  xvid->plugins[idx].func=func;
  xvid->plugins[idx].param=xvid->plugin_data[idx];
  xvid->create.num_plugins=idx+1;
}


void xvid_add_cbr_plugin(xvid_t *xvid, int bitrate) {
  xvid_plugin_single_t cbr;
  memset(&cbr,0,sizeof(xvid_plugin_single_t));
  cbr.version=XVID_VERSION;
  cbr.bitrate=bitrate*1000;
  xvid_add_plugin(xvid,xvid_plugin_single,&cbr,sizeof(xvid_plugin_single_t));
}


int xvid_create(xvid_t *xvid) {
  if(!xvid) return 0;
  if(xvid_encore(NULL,XVID_ENC_CREATE,&xvid->create,NULL)<0) return 0;
  xvid->handle=xvid->create.handle;
  return 1;
}


void xvid_set_me_preset(xvid_t *xvid, int preset) {
  if(!xvid || preset<0 || preset>=ME_PRESET_COUNT) return;
  xvid->me_preset=preset;
  xvid->frame.motion=me_presets[preset];
  xvid->frame.vop_flags=((preset>=HALFPEL_SINCE)?XVID_VOP_HALFPEL:0)
                       |((preset>=INTER4V_SINCE)?XVID_VOP_INTER4V:0);
}


void xvid_set_image(xvid_t *xvid,
                    uint8_t *Y, uint8_t *U, uint8_t *V,
                    int Ystride, int Cstride
) {
  if(!xvid) return;
  xvid->frame.input.csp=XVID_CSP_PLANAR;
  xvid->frame.input.plane[0]=Y; xvid->frame.input.stride[0]=Ystride;
  xvid->frame.input.plane[1]=U; xvid->frame.input.stride[1]=Cstride;
  xvid->frame.input.plane[2]=V; xvid->frame.input.stride[2]=Cstride;
}


uint8_t *xvid_encode_frame(xvid_t *xvid, int *size) {
  *size=0;
  if(!xvid) return NULL;
  xvid->frame.bitstream=xvid->bitstream;
  xvid->frame.length=BITSTREAM_BUFFER_SIZE;
  xvid->stats.version=XVID_VERSION;
  *size=xvid_encore(xvid->handle,XVID_ENC_ENCODE,&xvid->frame,&xvid->stats);
  if(*size<0) return NULL;
  return xvid->bitstream;
}


void xvid_destroy(xvid_t *xvid) {
  int i;
  if(!xvid) return;
  xvid_encore(xvid->handle,XVID_ENC_DESTROY,NULL,NULL);
  for(i=0; i<MAX_PLUGINS; ++i)
    if(xvid->plugin_data[i])
      free(xvid->plugin_data[i]);
}
