/*
Video encoding module (XviD wrapper) for lvstream.
lvstream (C) 2005 Martin J. Fiedler <martin.fiedler@gmx.net>

lvstream is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

lvstream is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lvstream; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __XVIDWRAP_H__
#define __XVIDWRAP_H__

#include <stdint.h>
#include "xvid.h"

#define ME_PRESET_COUNT 8
#define MAX_PLUGINS 10
#define BITSTREAM_BUFFER_SIZE (256*1024)

typedef struct {
  int me_preset;
  void *handle;
  xvid_enc_frame_t frame;
  xvid_enc_create_t create;
  xvid_enc_stats_t stats;
  xvid_enc_plugin_t plugins[MAX_PLUGINS];
  void* plugin_data[MAX_PLUGINS];
  uint8_t bitstream[BITSTREAM_BUFFER_SIZE];
} xvid_t;


char* xvid_init(void);

void xvid_prepare(xvid_t *xvid,
                  int width, int height,
                  int bframes, int key_interval,
                  int rate_num, int rate_den);
void xvid_add_plugin(xvid_t *xvid, xvid_plugin_func *func, void *param, int param_size);
void xvid_add_cbr_plugin(xvid_t *xvid, int bitrate);
int xvid_create(xvid_t *xvid);

void xvid_set_me_preset(xvid_t *xvid, int preset);

void xvid_set_image(xvid_t *xvid,
                    uint8_t *Y, uint8_t *U, uint8_t *V,
                    int Ystride, int Cstride);
uint8_t *xvid_encode_frame(xvid_t *xvid, int *size);

void xvid_destroy(xvid_t *xvid);

#endif/*__XVIDWRAP_H__*/
